@@ -7,7 +7,7 @@ class MVPoly(object) :
     Base (new-style) class for MVPoly. 
     """
     def __init__(self, dtype = np.double) :
-        self._dtype = dtype 
+        self._dtype = np.dtype(dtype) 
 
     def asclass(self, C) :
         """
@@ -22,7 +22,7 @@ class MVPoly(object) :
 
     @dtype.setter
     def dtype(self, value) :
-        self._dtype = value
+        self._dtype = np.dtype(value)
 
     def intd(self, *intervals) :
         """
