@@ -10,6 +10,7 @@
 
 import mvpoly.base
 import mvpoly.util.cube
+import mvpoly.util.dtype
 
 import numbers
 import numpy as np
@@ -27,11 +28,9 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         else :
             coef = arg
         super(MVPolyCube, self).__init__(**kwd)
-        if coef is None :
-            coef = np.array([], dtype = self.dtype)
-        else :
-            coef = np.array(coef, dtype = self.dtype)
-        self._coef = coef
+        self._coef = np.array([] if coef is None else coef, 
+                              dtype = self.dtype)
+
 
     @classmethod
     def init_from_nonzero(self, p, dtype) :
@@ -65,7 +64,7 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         (*i* = 0, ..., *n*-1).
         """
         shp = tuple((1 if j==i else 0) for j in range(n))
-        m = np.zeros([s+1 for s in shp])
+        m = np.zeros([s+1 for s in shp], **kwd)
         m.itemset(shp, 1)
         return MVPolyCube(m, **kwd)
 
@@ -92,7 +91,7 @@ class MVPolyCube(mvpoly.base.MVPoly) :
 
     @coef.setter
     def coef(self, value) :
-        self._coef = value
+        self._coef = np.array(value, dtype=self.dtype)
 
     def __getitem__(self, key) :
         """
@@ -158,15 +157,13 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         """
         a = self.coef
         if isinstance(other, MVPolyCube) :
-            b = other.coef
-        elif isinstance(other, list) :
-            b = np.array(other)
-        elif isinstance(other, numbers.Number) :
+            b = other.coef            
+        elif isinstance(other, (numbers.Number, np.generic)) :
             b = np.array([other])
         else :
             raise TypeError, "cannot add MVPolyCube to %s" % type(other)
-        return MVPolyCube(mvpoly.util.cube.padded_sum(a, b), 
-                          dtype = self.dtype)
+        dtype = mvpoly.util.dtype.dtype_add(a.dtype, b.dtype)
+        return MVPolyCube(mvpoly.util.cube.padded_sum(a, b), dtype = dtype)
 
     def __radd__(self, other) :
         """
@@ -182,13 +179,15 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         the result.
         """
         if isinstance(other, MVPolyCube) :
-            return MVPolyCube(mvpoly.util.cube.convn(self.coef, other.coef),
-                              dtype = self.dtype)
-        elif isinstance(other, numbers.Number) :
-            return MVPolyCube(self.coef * other, dtype = self.dtype)
+            coef  = mvpoly.util.cube.convn(self.coef, other.coef)
         else :
-            raise TypeError, \
-                "cannot multiply MVPolyCube by %s" % type(other)
+            if isinstance(other, numbers.Number) :
+                other = mvpoly.util.common.as_numpy_scalar(other)
+            if isinstance(other, np.generic) :
+                coef = self.coef * other
+            else :
+                raise TypeError, "cannot multiply MVPolyCube by %s" % type(other)
+        return MVPolyCube(coef, dtype=coef.dtype)
 
     def __rmul__(self, other) :
         """
@@ -272,11 +271,6 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         into the corresponding variables of the polynomial.
         """
 
-        # Note that this could be moved into the base class
-        # if we were to insist that each subclass had a 
-        # method enumerating the monomial indices and the
-        # corresponding non-zero coeficients, probably
-
         dims = self.coef.shape
         N = len(dims)
 
@@ -326,8 +320,8 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         hence ``p.diff(0,1,1)`` would correspond to 
         :math:`\partial^2 p / \partial y \partial z`.
         """
-        return MVPolyCube(mvpoly.util.cube.diff(self.coef, args),
-                          dtype = self.dtype)  
+        coef = mvpoly.util.cube.diff(np.atleast_1d(self.coef), args)
+        return MVPolyCube(coef, dtype=self.dtype)  
 
     def int(self, *args, **kwargs) :
         """
@@ -337,8 +331,8 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         specify the *dtype* of the output polynomial.
         """
         dtype = np.dtype(kwargs.get('dtype', self.dtype))
-        return MVPolyCube(mvpoly.util.cube.int(self.coef, args, dtype), 
-                          dtype=dtype)
+        coef  = mvpoly.util.cube.int(np.atleast_1d(self.coef), args, dtype)
+        return MVPolyCube(coef, dtype=dtype)
 
     def maxmodnb(self, **kwargs) :
         """
