@@ -34,9 +34,10 @@ class MVPolyDict(mvpoly.base.MVPoly) :
             coef = {}
         else :
             for k, v in coef.iteritems() :
-                coef[k] = np.array(v, dtype = self.dtype)
+                coef[k] = self.dtype.type(v)
         self._coef = coef
 
+
     @classmethod
     def init_from_nonzero(self, p, dtype) :
         """
@@ -50,6 +51,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
             q[idx] = val
         return q
 
+
     @classmethod
     def monomials(self, n, **kwd) :
         """
@@ -59,6 +61,20 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         return [self.monomial(i, n, **kwd) for i in range(n)]
 
 
+    @classmethod
+    def _numpy_scalar(self, v, **kwd) :
+        """
+        Return a :class:`numpy` scalar which, if the optional
+        argument contains a ``dtype`` entry, will be of that 
+        *dtype*.
+        """
+        dtype = kwd.get('dtype', False) 
+        if dtype :
+            return np.dtype(dtype).type(v)
+        else :
+            return np.int_(v)
+
+
     @classmethod
     def monomial(self, i, n, **kwd) :
         """
@@ -66,9 +82,10 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         (*i* = 0, ..., *n*-1).
         """
         m = MVPolyDictMonomial({i:1})
-        d = { m.key: np.array(1) }
+        d = { m.key : self._numpy_scalar(1, **kwd) }
         return MVPolyDict(d, **kwd)
 
+
     @classmethod
     def zero(self, **kwd) :
         """
@@ -76,15 +93,17 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         """
         return MVPolyDict(**kwd)
 
+
     @classmethod
     def one(self, **kwd) :
         """
         Return the unit (1) polynomial. 
         """
         m = MVPolyDictMonomial({})
-        d = { m.key: np.array(1) }
+        d = { m.key: self._numpy_scalar(1, **kwd) }
         return MVPolyDict(d, **kwd)
 
+
     @property
     def coef(self) :
         """
@@ -92,6 +111,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         """
         return self._coef
 
+
     @coef.setter
     def coef(self, value) :
         self._coef = value
@@ -105,7 +125,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         if value == 0 :
             return
         key = MVPolyDictMonomial(index = index).key
-        self.coef[key] = np.array(value, self.dtype)
+        self.coef[key] = self.dtype.type(value)
 
 
     def __getitem__(self, index) :
@@ -114,7 +134,8 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         specified *index*.
         """
         key = MVPolyDictMonomial(index = index).key
-        return self.coef.get(key, np.array(0, self.dtype))
+        return self.coef.get(key, self.dtype.type(0))
+
 
     def __add__(self, other) :
         """
@@ -122,15 +143,22 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         return an :class:`MVPolyDict`.
         """
         a = self.coef
+        a_dtype = self.dtype
         if isinstance(other, MVPolyDict) :
             b = other.coef
-        elif isinstance(other, numbers.Number) :
-            m = MVPolyDict()
-            m[0] = other
-            b = m.coef
+            b_dtype = other.dtype
         else :
-            raise TypeError, "cannot add MVPolyDict to %s" % type(other)
-        return MVPolyDict(mvpoly.util.dict.sum(a, b), dtype = self.dtype)
+            if isinstance(other, numbers.Number) :
+                other = mvpoly.util.common.as_numpy_scalar(other)
+            if isinstance(other, np.generic) :
+                m = MVPolyDict()
+                m[0] = other
+                b = m.coef
+                b_dtype = other.dtype
+            else :
+                raise TypeError, "cannot add MVPolyDict to %s" % type(other)
+        dtype = mvpoly.util.dtype.dtype_add(a_dtype, b_dtype)
+        return MVPolyDict(mvpoly.util.dict.sum(a, b), dtype = dtype)
 
 
     def __radd__(self, other) :
@@ -147,6 +175,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         """
         d = {}
         if isinstance(other, MVPolyDict) :
+            dtype = mvpoly.util.dtype.dtype_mul(self.dtype, other.dtype)
             for k1, v1 in other.coef.iteritems() :
                 m1 = MVPolyDictMonomial(key = k1)
                 for k2, v2 in self.coef.iteritems() :
@@ -157,14 +186,19 @@ class MVPolyDict(mvpoly.base.MVPoly) :
             for k, v in d.items() :                
                 if v == 0 :
                     d.pop(k, None)
-        elif isinstance(other, numbers.Number) :
-            if other != 0 :
-                for k, v in self.coef.iteritems() :
-                    d[k] = other * v
         else :
-            raise TypeError, \
-                "cannot multiply MVPolyDict by %s" % type(other)
-        return MVPolyDict(d, dtype = self.dtype)
+            if isinstance(other, numbers.Number) :
+                other = mvpoly.util.common.as_numpy_scalar(other)
+            if isinstance(other, np.generic) :
+                dtype = mvpoly.util.dtype.dtype_mul(self.dtype, other.dtype)
+                if other != 0 :
+                    for k, v in self.coef.iteritems() :
+                        d[k] = other * v
+            else :
+                raise TypeError, \
+                    "cannot multiply MVPolyDict by %s" % type(other)
+
+        return MVPolyDict(d, dtype=dtype)
 
 
     def __rmul__(self, other) :
@@ -187,7 +221,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         """
         Equality of polynomials. 
         """
-        return self.coef == other.coef
+        return not any((self - other).coef)
 
 
     def rmf_eval(self, *args) :
@@ -206,7 +240,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         """
         nodes = [EvalNode(k, v) for k, v in self.coef.items()]
         if self.__getitem__(0) == 0 :
-            nodes += [EvalNode((), np.array(0))]
+            nodes += [EvalNode((), np.int_(0))]
         nodes.sort(key = lambda x : x.index)
 
         deltas = [node1.delta(node2) 
@@ -299,9 +333,17 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         has variables.  The argument can be numbers, or arrays
         (all of the same shape).
         """
-        if len(set(np.array(arg).shape for arg in args)) != 1 :
+        # if the arguments are lists then we need to convert
+        # them to numpy arrays, since rmf_eval() requires that
+        array_args = [np.array(arg) for arg in args]
+        array_shps = set(arg.shape for arg in array_args)
+        if len(array_shps) != 1 :
             raise ValueError("eval() with arguments of differing shapes")
-        return self.rmf_eval(*args);
+        array_shp = array_shps.pop()
+        if array_shp == (1,) :
+            return self.rmf_eval(*args);
+        else :
+            return self.rmf_eval(*array_args);
 
 
     def compose(self, *args) :
@@ -375,6 +417,7 @@ class MVPolyDict(mvpoly.base.MVPoly) :
 
         return MVPolyDict(d, dtype = dtype)
 
+
 class EvalNode(object) :
     """
     Helper class for the :meth:`__call__` method.
