@@ -4,6 +4,17 @@ import unittest
 
 class TestMPUtil(unittest.TestCase):
 
+    def test_array_enlarge(self) :
+        A = np.array( [[1, 2],
+                       [3, 4]], dtype = int)
+        shape = (3, 4)
+        B = mvpoly.util.cube.array_enlarge(A, shape)
+        self.assertTrue((B == [[1,2,0,0],
+                               [3,4,0,0],
+                               [0,0,0,0]]).all(),
+                        "bad enlarge:\n%s" % repr(B))
+        self.assertTrue(B.dtype == B.dtype, "enlarge dtype")
+
     def test_padded_sum(self) :
         A = np.array([1, 2, 3])
         B = np.array([[1], [1]])
